==> json-joy-2.15.0.tgz: package/esm/reactive-rpc/common/rpc/RpcServer.js <==
import { Subject } from 'rxjs';
import { NotificationMessage, RequestCompleteMessage, RequestDataMessage, RequestErrorMessage, RequestUnsubscribeMessage, ResponseCompleteMessage, ResponseDataMessage, ResponseErrorMessage, ResponseUnsubscribeMessage } from '../messages/nominal';
import { subscribeCompleteObserver } from '../util/subscribeCompleteObserver';
import { TimedQueue } from '../util/TimedQueue';
import { RpcServerError } from './constants';
import { ErrorLikeErrorFormatter, RpcError } from './error';
export { RpcServerError };
class StreamCall {
    constructor() {
        this.reqFinalized = false;
        this.resFinalized = false;
        this.req$ = new Subject();
        this.res$ = new Subject();
    }
}
export class RpcServer {
    constructor({ caller, error, send, onNotification: notify, bufferSize = 10, bufferTime = 1, }) {
        this.activeStreamCalls = new Map();
        this.caller = caller;
        this.error = error || new ErrorLikeErrorFormatter();
        this.onNotification = notify;
        this.onSend = send;
        if (bufferTime) {
            const buffer = new TimedQueue();
            buffer.itemLimit = bufferSize;
            buffer.timeLimit = bufferTime;
            buffer.onFlush = messages => this.onSend(messages);
            this.send = (message) => {
                buffer.push(message);
            };
        }
        else {
            this.send = message => {
                this.onSend([message]);
            };
        }
    }
    onMessage(message, ctx) {
        try {
            if (message instanceof RequestDataMessage)
                this.onRequestDataMessage(message, ctx);
            else if (message instanceof RequestCompleteMessage)
                this.onRequestCompleteMessage(message, ctx);
            else if (message instanceof RequestErrorMessage)
                this.onRequestErrorMessage(message, ctx);
            else if (message instanceof NotificationMessage)
                this.onNotificationMessage(message, ctx);
            else if (message instanceof ResponseUnsubscribeMessage)
                this.onUnsubscribeMessage(message);
        }
        catch (error) {
            const formattedError = this.error.format(error);
            const message = new NotificationMessage('.err', formattedError);
            this.send(message);
        }
    }
    onMessages(messages, ctx) {
        const length = messages.length;
        for (let i = 0; i < length; i++)
            this.onMessage(messages[i], ctx);
    }
    stop(reason = RpcServerError.Stop) {
        this.send = (message) => { };
        this.onNotification = (name, data, ctx) => { };
        for (const call of this.activeStreamCalls.values()) {
            call.req$.error(new RpcError(reason));
        }
        this.activeStreamCalls.clear();
    }
    disconnect() {
        this.stop(RpcServerError.Disconnect);
    }
    sendError(id, code) {
        const data = this.error.formatCode(code);
        const message = new ResponseErrorMessage(id, data);
        this.send(message);
    }
    execStaticCall(id, name, request, ctx) {
        this.caller.call(name, request, ctx)
            .then(response => {
            this.send(new ResponseCompleteMessage(id, response));
        })
            .catch(error => {
            const formattedError = this.error.format(error);
            this.send(new ResponseErrorMessage(id, formattedError));
        });
    }
    createStreamCall(id, name, ctx) {
        const call = this.caller.createCall(name, ctx);
        this.activeStreamCalls.set(id, call);
        subscribeCompleteObserver(call.res$, {
            next: (value) => {
                this.send(new ResponseDataMessage(id, value));
            },
            error: (error) => {
                this.send(new ResponseErrorMessage(id, this.error.format(error)));
                this.activeStreamCalls.delete(id);
            },
            complete: (value) => {
                this.send(new ResponseCompleteMessage(id, value));
                this.activeStreamCalls.delete(id);
            },
        });
        call.reqUnsubscribe$.subscribe(() => {
            if (this.activeStreamCalls.has(id))
                this.send(new RequestUnsubscribeMessage(id));
        });
        return call;
    }
    onRequestDataMessage(message, ctx) {
        const { id, method, data } = message;
        let call = this.activeStreamCalls.get(id);
        if (!call) {
            if (!method) {
                this.sendError(id, RpcServerError.NoMethodSpecified);
                return;
            }
            call = this.createStreamCall(id, method, ctx);
        }
        if (call) {
            if (data !== undefined) {
                call.req$.next(data);
            }
        }
    }
    onRequestCompleteMessage(message, ctx) {
        const { id, method, data } = message;
        const call = this.activeStreamCalls.get(id);
        if (call) {
            const { req$ } = call;
            if (data !== undefined)
                req$.next(data);
            req$.complete();
            return;
        }
        if (!method) {
            this.sendError(id, RpcServerError.NoMethodSpecified);
            return;
        }
        if (!this.caller.exists(method)) {
            this.sendError(id, RpcServerError.MethodNotFound);
            return;
        }
        const rpcMethod = this.caller.get(method);
        if (rpcMethod.isStreaming) {
            const newCall = this.createStreamCall(id, method, ctx);
            if (newCall) {
                if (data !== undefined) {
                    newCall.req$.next(data);
                }
            }
        }
        else
            this.execStaticCall(id, method, data, ctx);
    }
    receiveRequestData(method, call, data) {
        if (data === undefined)
            return;
        if (method.validate) {
            try {
                method.validate(data);
            }
            catch (error) {
                call.req$.error(error);
                call.res$.error(error);
                return;
            }
        }
        call.req$.next(data);
    }
    onRequestErrorMessage(message, ctx) {
        const { id, method, data } = message;
        const call = this.activeStreamCalls.get(id);
        if (call) {
            return call.req$.error(data);
        }
        if (!method)
            return this.sendError(id, RpcServerError.NoMethodSpecified);
        if (!this.caller.exists(method))
            return this.sendError(id, RpcServerError.MethodNotFound);
        const rpcMethod = this.caller.get(method);
        if (!rpcMethod.isStreaming)
            return this.sendError(id, RpcServerError.ErrorForStaticMethod);
        const streamCall = this.createStreamCall(id, rpcMethod, ctx);
        if (!streamCall)
            return;
        streamCall.req$.error(data);
    }
    onUnsubscribeMessage(message) {
        const { id } = message;
        const call = this.activeStreamCalls.get(id);
        if (!call)
            return;
        this.activeStreamCalls.delete(id);
        call.req$.complete();
    }
    onNotificationMessage(message, ctx) {
        const { method, data } = message;
        if (!method || (method.length > 128))
            throw new RpcError(RpcServerError.InvalidNotificationName);
        this.onNotification(method, data, ctx);
    }
}

==> json-joy-2.16.0.tgz: package/esm/reactive-rpc/common/rpc/RpcServer.js <==
import { Subject } from 'rxjs';
import { NotificationMessage, RequestCompleteMessage, RequestDataMessage, RequestErrorMessage, RequestUnsubscribeMessage, ResponseCompleteMessage, ResponseDataMessage, ResponseErrorMessage, ResponseUnsubscribeMessage } from '../messages/nominal';
import { subscribeCompleteObserver } from '../util/subscribeCompleteObserver';
import { TimedQueue } from '../util/TimedQueue';
import { RpcServerError } from './constants';
import { ErrorLikeErrorFormatter, RpcError } from './error';
export { RpcServerError };
class StreamCall {
    constructor() {
        this.reqFinalized = false;
        this.resFinalized = false;
        this.req$ = new Subject();
        this.res$ = new Subject();
    }
}
export class RpcServer {
    constructor({ caller, error, send, onNotification: notify, bufferSize = 10, bufferTime = 1, }) {
        this.activeStreamCalls = new Map();
        this.caller = caller;
        this.error = error || new ErrorLikeErrorFormatter();
        this.onNotification = notify;
        this.onSend = send;
        if (bufferTime) {
            const buffer = new TimedQueue();
            buffer.itemLimit = bufferSize;
            buffer.timeLimit = bufferTime;
            buffer.onFlush = messages => this.onSend(messages);
            this.send = (message) => {
                buffer.push(message);
            };
        }
        else {
            this.send = message => {
                this.onSend([message]);
            };
        }
    }
    onMessage(message, ctx) {
        try {
            if (message instanceof RequestDataMessage)
                this.onRequestDataMessage(message, ctx);
            else if (message instanceof RequestCompleteMessage)
                this.onRequestCompleteMessage(message, ctx);
            else if (message instanceof RequestErrorMessage)
                this.onRequestErrorMessage(message, ctx);
            else if (message instanceof NotificationMessage)
                this.onNotificationMessage(message, ctx);
            else if (message instanceof ResponseUnsubscribeMessage)
                this.onUnsubscribeMessage(message);
        }
        catch (error) {
            const formattedError = this.error.format(error);
            const message = new NotificationMessage('.err', formattedError);
            this.send(message);
        }
    }
    onMessages(messages, ctx) {
        const length = messages.length;
        for (let i = 0; i < length; i++)
            this.onMessage(messages[i], ctx);
    }
    stop(reason = RpcServerError.Stop) {
        this.send = (message) => { };
        this.onNotification = (name, data, ctx) => { };
        for (const call of this.activeStreamCalls.values()) {
            call.req$.error(new RpcError(reason));
        }
        this.activeStreamCalls.clear();
    }
    disconnect() {
        this.stop(RpcServerError.Disconnect);
    }
    sendError(id, code) {
        const data = this.error.formatCode(code);
        const message = new ResponseErrorMessage(id, data);
        this.send(message);
    }
    execStaticCall(id, name, request, ctx) {
        this.caller.call(name, request, ctx)
            .then(response => {
            this.send(new ResponseCompleteMessage(id, response));
        })
            .catch(error => {
            this.send(new ResponseErrorMessage(id, error));
        });
    }
    createStreamCall(id, name, ctx) {
        const call = this.caller.createCall(name, ctx);
        this.activeStreamCalls.set(id, call);
        subscribeCompleteObserver(call.res$, {
            next: (value) => {
                this.send(new ResponseDataMessage(id, value));
            },
            error: (error) => {
                this.send(new ResponseErrorMessage(id, error));
                this.activeStreamCalls.delete(id);
            },
            complete: (value) => {
                this.send(new ResponseCompleteMessage(id, value));
                this.activeStreamCalls.delete(id);
            },
        });
        call.reqUnsubscribe$.subscribe(() => {
            if (this.activeStreamCalls.has(id))
                this.send(new RequestUnsubscribeMessage(id));
        });
        return call;
    }
    onRequestDataMessage(message, ctx) {
        const { id, method, data } = message;
        let call = this.activeStreamCalls.get(id);
        if (!call) {
            if (!method) {
                this.sendError(id, RpcServerError.NoMethodSpecified);
                return;
            }
            call = this.createStreamCall(id, method, ctx);
        }
        if (call) {
            if (data !== undefined) {
                call.req$.next(data);
            }
        }
    }
    onRequestCompleteMessage(message, ctx) {
        const { id, method, data } = message;
        const call = this.activeStreamCalls.get(id);
        if (call) {
            const { req$ } = call;
            if (data !== undefined)
                req$.next(data);
            req$.complete();
            return;
        }
        if (!method) {
            this.sendError(id, RpcServerError.NoMethodSpecified);
            return;
        }
        if (!this.caller.exists(method)) {
            this.sendError(id, RpcServerError.MethodNotFound);
            return;
        }
        const rpcMethod = this.caller.get(method);
        if (rpcMethod.isStreaming) {
            const newCall = this.createStreamCall(id, method, ctx);
            if (newCall) {
                if (data !== undefined) {
                    newCall.req$.next(data);
                }
            }
        }
        else
            this.execStaticCall(id, method, data, ctx);
    }
    receiveRequestData(method, call, data) {
        if (data === undefined)
            return;
        if (method.validate) {
            try {
                method.validate(data);
            }
            catch (error) {
                call.req$.error(error);
                call.res$.error(error);
                return;
            }
        }
        call.req$.next(data);
    }
    onRequestErrorMessage(message, ctx) {
        const { id, method, data } = message;
        const call = this.activeStreamCalls.get(id);
        if (call) {
            return call.req$.error(data);
        }
        if (!method)
            return this.sendError(id, RpcServerError.NoMethodSpecified);
        if (!this.caller.exists(method))
            return this.sendError(id, RpcServerError.MethodNotFound);
        const rpcMethod = this.caller.get(method);
        if (!rpcMethod.isStreaming)
            return this.sendError(id, RpcServerError.ErrorForStaticMethod);
        const streamCall = this.createStreamCall(id, rpcMethod, ctx);
        if (!streamCall)
            return;
        streamCall.req$.error(data);
    }
    onUnsubscribeMessage(message) {
        const { id } = message;
        const call = this.activeStreamCalls.get(id);
        if (!call)
            return;
        this.activeStreamCalls.delete(id);
        call.req$.complete();
    }
    onNotificationMessage(message, ctx) {
        const { method, data } = message;
        if (!method || (method.length > 128))
            throw new RpcError(RpcServerError.InvalidNotificationName);
        this.onNotification(method, data, ctx);
    }
}
